/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';

import {classNames, useFocusableRef, useStyleProps} from '@react-spectrum/utils';
import DashSmall from '@spectrum-icons/ui/DashSmall';
import {FocusableRef} from '@react-ui-types/shared';
import {FocusRing} from '@react-aria/focus';
import {SpectrumCheckboxProps} from '@react-ui-types/checkbox';
import {useCheckbox} from '@react-aria/checkbox';
import {useHover} from '@react-aria/interactions';
import {useProviderProps} from '@ladifire-ui-react/provider';
import {useToggleState} from '@react-stately/toggle';
import {CheckMarkIcon} from '@ladifire-ui-react/icons';

import gs from '@ladifire-css/sass/index.scss';
import checkboxStyles from '@ladifire-css/sass/components/checkbox.scss';

function Checkbox(props: SpectrumCheckboxProps, ref: FocusableRef<HTMLLabelElement>) {
  props = useProviderProps(props);
  let {
    isIndeterminate = false,
    isEmphasized = false,
    isDisabled = false,
    autoFocus,
    children,
    disabledHoverBackground = false,
    ...otherProps
  } = props;
  let {styleProps} = useStyleProps(otherProps);
  let {hoverProps, isHovered} = useHover(props);

  let inputRef = React.useRef<HTMLInputElement>(null);
  let domRef = useFocusableRef(ref, inputRef);
  let state = useToggleState(props);
  let {inputProps} = useCheckbox(props, state, inputRef);

  let markIcon = isIndeterminate
    ? <DashSmall UNSAFE_className='spectrum-Checkbox-partialCheckmark' />
    : <CheckMarkIcon size={18} className={classNames(
      gs,
      checkboxStyles.check_mark,
      gs.display__inline_block,
      gs.pointer_event__none,
      gs.fill_current,
      gs.width__18,
      gs.height__18,
      gs.position__absolute,
      gs.color__white,
      gs.opacity__0, {
        [gs.opacity__1]: state.isSelected,
      }
    )} />;

  return (
    <label
      {...styleProps}
      {...hoverProps}
      ref={domRef}
      className={classNames(
        gs,
        gs.display__inline_flex,
        gs.box_sizing__border_box,
        gs.align_item__flex_start,
        gs.position__relative,
        gs.vertical_align__top,
        gs.max_width__100_percent,
        gs.isolation__isolate,
        styleProps.className,
      )}
      // className={
      //   classNames(
      //     styles,
      //     'spectrum-Checkbox',
      //     {
      //       'is-checked': state.isSelected,
      //       'is-indeterminate': isIndeterminate,
      //       'spectrum-Checkbox--quiet': !isEmphasized,
      //       'is-invalid': inputProps['aria-invalid'],
      //       'is-disabled': isDisabled,
      //       'is-hovered': isHovered
      //     },
      //     styleProps.className
      //   )
      // }
    >
      <FocusRing focusRingClass='focus-ring' autoFocus={autoFocus}>
        <input
          {...inputProps}
          ref={inputRef}
          className={
            classNames(
              gs,
              gs.margin__0,
              gs.overflow__visible,
              gs.box_sizing__border_box,
              gs.padding_0,
              gs.position__absolute,
              gs.top__0,
              gs.height__100_percent,
              gs.z_index__1,
              gs.cursor__pointer,
              gs.opacity__00001,
              gs.width__100_percent, {
                [gs.pointer_event__none]: isDisabled,
              }
            )
          } />
      </FocusRing>
      <span
        className={classNames(
          gs,
          gs.position__relative,
          gs.box_sizing__border_box,
          gs.flex_grow__0,
          gs.flex_grow__1,
          gs.ms_flex_negative__0,
          gs.ms_flex_positive__0,
          gs.height__18,
          gs.width__18,
          checkboxStyles.checkbox_box, {
            [checkboxStyles.checkbox_box__checked]: state.isSelected,
            [gs.opacity__05]: isDisabled,
        }
      )}>
        {markIcon}
        {
          !disabledHoverBackground && (
            <div
              className={classNames(
                gs,
                gs.position__absolute,
                gs.top__0,
                gs.left__0,
                gs.right__0,
                gs.bottom__0,
                gs.opacity__0,
                gs.transition_timing_function__fade_out,
                gs.pointer_event__none,
                gs.transition_duration__short_out,
                gs.transition_property__opacity,
                gs.cursor__pointer,
                gs.border_radius__50_percent,
                gs.background_color__hover_overlay,
                gs.margin_left__8m,
                gs.margin_right__8m,
                gs.margin_top__8m,
                gs.margin_bottom__8m,
                gs.pointer_event__none, {
                  [gs.opacity__1]: isHovered,
                }
              )}
            />
          )
        }
      </span>
      {children && (
        <span className={classNames(
          gs,
          gs.text_align__start,
          gs.margin_left__12, {
            [gs.opacity__05]: isDisabled,
          }
        )}>
          {children}
        </span>
      )}
    </label>
  );
}
/**
 * Checkboxes allow users to select multiple items from a list of individual items,
 * or to mark one individual item as selected.
 */
let _Checkbox = React.forwardRef(Checkbox);
export {_Checkbox as Checkbox};
